import 'package:dio_cache_interceptor/dio_cache_interceptor.dart';
import 'package:dio_cache_interceptor_hive_store/dio_cache_interceptor_hive_store.dart';
import 'package:flutter/foundation.dart';
import 'package:os_basecode/os_basecode.dart';

class Client {
  String baseURL;
  Map<String, dynamic>? headers;
  bool showLogging;
  bool useCache;
  Function? handleWhenUnauthorized;
  Function(RequestOptions requestOptions)? handleOnRequest;
  Function(Response response)? handleOnResponse;
  Duration? connectTimeout;
  Duration? receiveTimeout;
  Duration? maxStaleCache;
  List<int>? hitCacheOnErrorExcept;
  Client(
      {required this.baseURL,
      this.headers,
      this.showLogging = true,
      this.handleWhenUnauthorized,
      this.connectTimeout,
      this.receiveTimeout,
      this.hitCacheOnErrorExcept,
      this.maxStaleCache,
      this.handleOnRequest,
      this.handleOnResponse,
      this.useCache = false});

  Dio init() {
    var dio = Dio();
    dio.options.baseUrl = baseURL;
    dio.options.connectTimeout = connectTimeout ?? 120.seconds;
    dio.options.receiveTimeout = receiveTimeout ?? 120.seconds;
    dio.options.headers = headers;
    if (useCache) {
      if (!kIsWeb) {
        dio.interceptors.add(cacheInterceptor(
            maxStale: maxStaleCache, hitCacheOnErrorExcept: hitCacheOnErrorExcept));
      }
    }
    dio.interceptors.add(ApiInterceptors(
        handleOnRequest: handleOnRequest ?? (value) {},
        handleResponse: handleOnResponse ?? (value) {},
        showLogging: showLogging,
        handleWhenUnauthorized: handleWhenUnauthorized ?? () {}));
    return dio;
  }
}

DioCacheInterceptor cacheInterceptor({Duration? maxStale, List<int>? hitCacheOnErrorExcept}) {
  return DioCacheInterceptor(
      options: CacheOptions(
          store: HiveCacheStore(AppPathProvider.path),
          policy: CachePolicy.refreshForceCache,
          hitCacheOnErrorExcept: hitCacheOnErrorExcept,
          maxStale: maxStale ?? 7.days,
          priority: CachePriority.high));
}

class ApiInterceptors extends Interceptor {
  bool showLogging;
  Function handleWhenUnauthorized;
  Function(RequestOptions options) handleOnRequest;
  Function(Response options) handleResponse;

  ApiInterceptors(
      {this.showLogging = true,
      required this.handleWhenUnauthorized,
      required this.handleResponse,
      required this.handleOnRequest});
  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    if (showLogging) {
      if (kDebugMode) {
        print('OS BASECODE - REQUEST[${options.method}] => PATH: ${options.path}');
        print('OS BASECODE - DATA BODY: [${options.data}]');
        handleOnRequest(options);
      }
    }
    return super.onRequest(options, handler);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    if (showLogging) {
      if (kDebugMode) {
        print(
            'OS BASECODE - RESPONSE[${response.statusCode}] => PATH: ${response.requestOptions.path}');
        if (response.extra['cacheResponse'] != null) {
          print('OS BASECODE - Data from cache');
        } else {
          print('OS BASECODE - Data from server');
        }
        handleResponse(response);
      }
    }
    return super.onResponse(response, handler);
  }

  @override
  void onError(DioException err, ErrorInterceptorHandler handler) async {
    if (showLogging) {
      if (kDebugMode) {
        print(
            'OS BASECODE - ERROR[${err.response?.statusCode}] => PATH: ${err.requestOptions.path}');
        print('OS BASECODE - FULL ERROR: ${err.toString()}');
        print('OS BASECODE - ERROR MESSAGE : ${err.message}');
        print('OS BASECODE - ERROR TRACE : ${err.stackTrace}');
      }
    }
    if (err.response?.statusCode == 401) {
      handleWhenUnauthorized();
    }

    return super.onError(err, handler);
  }
}
